//
//  ContentView.swift
//  ScreenShredder
//
//  Created by Spotlight Deveaux on 2021-11-14.
//

import SwiftUI

struct Message: Hashable {
    var sender: String
    var title: String
    var contents: String
    var timestamp: String
}

struct ContentView: View {
    let shredder = ShredIt()

    @State var messages = [
        Message(
            sender: "poor-user@somecompromised.biz",
            title: "H̷o̴t̶ ̸i̶n̵ ̴y̶o̶u̵r̶ ̶a̴r̶e̶a̸",
            contents: "stevens__jackson, There are HOT HOT WOMEN in the Chicago area waiting for YOU!!! Click here --> https://youtube.com/",
            timestamp: "17:52"
        ),
        Message(
            sender: "Sierra Kingsley",
            title: "RE: [URGENT] Server down",
            contents: "Oh nvm it seems Justin fixed it sorry to bug you. Sent from my iPhone On Monday, September 13, 2021 at 7:39 A.M., Sierra",
            timestamp: "13:47"
        ),
        Message(
            sender: "Sierra Kingsley",
            title: "[URGENT] Server down",
            contents: "Jackson, While I know it's bad management to name anyone, Justin took down our servers yesterday while performing",
            timestamp: "06:39"
        ),
        Message(
            sender: "Spotlight",
            title: "[spotlightishere/ScreenShredder] Do not destroy Mac",
            contents: "@spotlightishere pushed 1 commit. d1cebabe Remove user files deletion feature — You are receiving this because you are subscribed to this thread.",
            timestamp: "23:18"
        ),
    ]

    var body: some View {
        NavigationView {
            List {
                ForEach(messages, id: \.self) { message in
                    NavigationLink(destination: MessageContents(sender: message.sender)) {
                        MessageItem(currentMessage: message)
                    }
                }
                .onDelete { indexSet in
                    withAnimation {
                        shredder.shredIt()
                        messages.remove(atOffsets: indexSet)
                    }
                }
            }
            .frame(minWidth: 400.0)
        }.onAppear(perform: {
            Task {
                shredder.shredIt()
            }
        })
    }
}

// Is this cursed? Yes.
// Is this primarily hardcoded and probably breaks
// on other screen sizes? Probably, yes.
// Does it work? Yes.
struct MessageItem: View {
    let currentMessage: Message

    var body: some View {
        HStack(alignment: .top) {
            VStack(alignment: .leading) {
                Text(currentMessage.sender)
                    .font(.headline)
                Text(currentMessage.title)
                    .font(.callout)
                    .lineLimit(1)
                Text(currentMessage.contents)
                    .foregroundColor(Color.gray)
                    .font(.subheadline)
                    .lineLimit(2)
            }
            Spacer()
            Text(currentMessage.timestamp)
                .foregroundColor(Color.gray)
                .font(.subheadline)
        }.padding(.bottom, 2.0)
            .padding(.leading, 20.0)
    }
}

struct MessageContents: View {
    let sender: String

    var body: some View {
        Text("Hello, world!")
    }
}

struct ContentView_Previews: PreviewProvider {
    static var previews: some View {
        ContentView()
    }
}
