//
//  ShredAroundAndFindOut.swift
//  ScreenShredder
//
//  Created by Spotlight Deveaux on 2021-11-14.
//

import Cocoa
import Foundation

class ShredIt: CAStateControllerDelegate {
    func shredIt() {
        let keyWindow = NSApplication.shared.keyWindow!
        let pdfData = keyWindow.dataWithPDF(inside: keyWindow.frame)

        let windowImage = NSImage(data: pdfData)!
        let shreddedImage = windowImage.cgImage(forProposedRect: nil, context: nil, hints: nil)
        let shreddedImageFlipped = flipHorizontally(windowImage).cgImage(forProposedRect: nil, context: nil, hints: nil)

        let view = keyWindow.contentView!

        // Here we, uh, go. I think.
        // Oh no...
        let caarPath = getBundleBase().appendingPathComponent("ShredderSheet.caar")

        do {
            // Load package
            let package = try CAPackage(contentsOf: caarPath, type: kCAPackageTypeArchive, options: nil)
            // It seems we need to add the root layer as provided by the package to us.
            let rootLayer = package.rootLayer
            let rootLayerFrame = getHorizontalCenter(view)

            // We can create a view with this layer.
            let layerView = NSView(frame: rootLayerFrame)
            layerView.layer = CALayer()
            layerView.layer?.addSublayer(rootLayer)
            view.addSubview(layerView)
            package.publishedObject(withName: "Root")!.frame = rootLayerFrame

            // Apparently allows us to control states within the animation.
            let stateController = CAStateController(layer: rootLayer)
            stateController.delegate = self

            // TODO: How do these differ?
            // These names are derived from PKPassShredderSheet's getter/setters.
            // let offscreenState = rootLayer.states[0]
            // let upOnBackState = rootLayer.states[1]
            let shreddingState = rootLayer.states[2]
            // The following names are guesses.
            // let hangingShreddingState = rootLayer.states[3]
            // let hangingShreddingShorterState = rootLayer.states[4]
            // let blankState = rootLayer.states[5]

            stateController.setInitialStatesOf(rootLayer, transitionSpeed: 1.0)
            stateController.setState(shreddingState, of: rootLayer, transitionSpeed: 1.0)

            // For debugging purposes
            print(package.publishedObjectNames())

            // Ensure we don't have some weird black background.
            package.publishedObject(withName: "Root")!.backgroundColor = NSColor.clear.cgColor

            // We never use the states (offscreenState/onBackState) where these are present.
            // We'll set them to a clear color, just in case.
            package.publishedObject(withName: "cancelButton")!.contents = NSColor.clear.cgColor
            package.publishedObject(withName: "deleteButton")!.contents = NSColor.clear.cgColor
            package.publishedObject(withName: "shredderPrompt")!.backgroundColor = NSColor.clear.cgColor

            // Have our wonderful, extremely 2011/2012-esque metal design.
            package.publishedObject(withName: "shredderFront")!.contents = getImage(named: "ShredderMetal")
            package.publishedObject(withName: "shredderBack")!.contents = getImage(named: "ShredderMetal")

            // Set what we're shredding.
            package.publishedObject(withName: "cardFrontUnshredded")!.contents = shreddedImage
            package.publishedObject(withName: "cardFrontShredded")!.contents = shreddedImage
            package.publishedObject(withName: "cardBackUnshredded")!.contents = shreddedImageFlipped

            // Passbook appears to obtain the least emitter cell and recolor to match the pass's color.
            // TODO: implement this
            // let lastEmitterCell = (package.publishedObject(withName: "shardEmitterLayer") as! CAEmitterLayer).emitterCells!.last

            //        print(package.publishedObject(withName: "cardFrontShredded"))
            //        package.publishedObject(withName: "cardBackShredded")!.contents = shreddedImageFlipped
        } catch {
            // Not my fault, this code is heavily tested
            print(error)
        }
    }

    func getBundleBase() -> URL {
        URL(fileURLWithPath: "/Users/spot/Downloads/PassKit.framework")
    }

    func getImage(named name: String) -> CGImage {
        let imagePath = getBundleBase().appendingPathComponent("\(name)@2x.png")
        let image = NSImage(contentsOf: imagePath)!
        return image.cgImage(forProposedRect: nil, context: nil, hints: nil)!
    }

    func getHorizontalCenter(_ view: NSView) -> NSRect {
        let frame = view.frame
        return NSRect(x: frame.midX / 4, y: frame.midY / 4, width: frame.width, height: frame.height)
    }

    // Shamelessly stolen from https://gist.github.com/rishigesh/fc66a2cb35372b2aa011a7a69f606d40
    // I'm too tired to think
    public func flipHorizontally(_ existingImage: NSImage) -> NSImage {
        let existingSize = existingImage.size
        let newSize = NSMakeSize(existingSize.width, existingSize.height)
        let flipedImage = NSImage(size: newSize)
        flipedImage.lockFocus()

        let t = NSAffineTransform()
        t.translateX(by: existingSize.width, yBy: 0.0)
        t.scaleX(by: -1.0, yBy: 1.0)
        t.concat()

        let rect = NSRect(x: 0, y: 0, width: newSize.width, height: newSize.height)
        existingImage.draw(at: .zero, from: rect, operation: .sourceOver, fraction: 1.0)
        flipedImage.unlockFocus()
        return flipedImage
    }
}
